@@ -236,7 +236,14 @@ class SQLGenerator:
         cte_sqls = []
         for model_name in all_models:
             model_filters = pushdown_filters.get(model_name, [])
-            cte_sql = self._build_model_cte(model_name, parsed_dims, metrics, model_filters if model_filters else None)
+            cte_sql = self._build_model_cte(
+                model_name,
+                parsed_dims,
+                metrics,
+                model_filters if model_filters else None,
+                order_by=order_by,
+                all_models=all_models,
+            )
             cte_sqls.append(cte_sql)
 
         # Build main SELECT using builder API (only with filters that couldn't be pushed down)
@@ -455,12 +462,63 @@ class SQLGenerator:
 
         return pushdown_filters, main_query_filters
 
+    def _find_needed_dimensions(
+        self,
+        model_name: str,
+        dimensions: list[tuple[str, str | None]],
+        filters: list[str] | None,
+        order_by: list[str] | None,
+    ) -> set[str]:
+        """Find which dimensions from this model are actually needed.
+
+        Args:
+            model_name: Model to check
+            dimensions: Parsed dimension references from query
+            filters: Filter expressions
+            order_by: Order by fields
+
+        Returns:
+            Set of dimension names needed for this model
+        """
+        needed = set()
+
+        # Dimensions explicitly in SELECT
+        for dim_ref, _ in dimensions:
+            if dim_ref.startswith(model_name + "."):
+                dim_name = dim_ref.split(".")[1]
+                needed.add(dim_name)
+
+        # Dimensions referenced in filters
+        if filters:
+            for filter_expr in filters:
+                try:
+                    parsed = sqlglot.parse_one(filter_expr, dialect=self.dialect)
+                    for col in parsed.find_all(exp.Column):
+                        if col.table and col.table.replace("_cte", "") == model_name:
+                            needed.add(col.name)
+                except Exception:
+                    pass
+
+        # Dimensions referenced in ORDER BY
+        if order_by:
+            for order_field in order_by:
+                # Strip DESC/ASC
+                field = order_field.replace(" DESC", "").replace(" ASC", "").strip()
+                if "." in field:
+                    model_part, dim_part = field.split(".", 1)
+                    if model_part == model_name:
+                        needed.add(dim_part)
+
+        return needed
+
     def _build_model_cte(
         self,
         model_name: str,
         dimensions: list[tuple[str, str | None]],
         metrics: list[str],
         filters: list[str] | None = None,
+        order_by: list[str] | None = None,
+        all_models: set[str] | None = None,
     ) -> str:
         """Build CTE SQL for a model with optional filter pushdown.
 
@@ -469,49 +527,69 @@ class SQLGenerator:
             dimensions: Parsed dimension references
             metrics: Metric references
             filters: Filters to push down into this CTE (optional)
+            order_by: Order by fields (for determining needed dimensions)
+            all_models: All models in query (for determining if joins needed)
 
         Returns:
             CTE SQL string
         """
         model = self.graph.get_model(model_name)
+        all_models = all_models or {model_name}
+        needs_joins = len(all_models) > 1
+
+        # Find which dimensions are actually needed
+        needed_dimensions = self._find_needed_dimensions(model_name, dimensions, filters, order_by)
 
         # Build SELECT columns
         select_cols = []
 
-        # Add join keys
-        join_keys_added = set()
+        # Track all columns added (not just join keys) to avoid duplicates
+        columns_added = set()
 
-        # Include this model's primary key
-        if model.primary_key and model.primary_key not in join_keys_added:
+        # Include this model's primary key (always needed for joins/grouping)
+        if model.primary_key and model.primary_key not in columns_added:
             select_cols.append(f"{model.primary_key} AS {model.primary_key}")
-            join_keys_added.add(model.primary_key)
+            columns_added.add(model.primary_key)
 
-        # Include foreign keys from belongs_to joins
+        # Include foreign keys if we're joining OR if they're explicitly requested as dimensions
         for relationship in model.relationships:
             if relationship.type == "many_to_one":
                 fk = relationship.sql_expr
-                if fk not in join_keys_added:
+                # Add FK if: (1) we're joining to this related model, OR (2) FK is requested as dimension
+                should_include = (needs_joins and relationship.name in all_models) or fk in needed_dimensions
+                if should_include and fk not in columns_added:
                     select_cols.append(f"{fk} AS {fk}")
-                    join_keys_added.add(fk)
+                    columns_added.add(fk)
+                    # Mark FK as "needed" so it's not duplicated as a dimension
+                    needed_dimensions.discard(fk)
 
         # Check if other models have has_many/has_one pointing to this model
-        for other_model_name, other_model in self.graph.models.items():
-            for other_join in other_model.relationships:
-                if other_join.name == model_name and other_join.type in (
-                    "one_to_one",
-                    "one_to_many",
-                ):
-                    # Other model expects this model to have a foreign key
-                    # For has_many/has_one, foreign_key is the FK column in THIS model
-                    fk = other_join.foreign_key or other_join.sql_expr
-                    if fk not in join_keys_added:
-                        select_cols.append(f"{fk} AS {fk}")
-                        join_keys_added.add(fk)
+        if needs_joins:
+            for other_model_name, other_model in self.graph.models.items():
+                if other_model_name not in all_models:
+                    continue
+                for other_join in other_model.relationships:
+                    if other_join.name == model_name and other_join.type in (
+                        "one_to_one",
+                        "one_to_many",
+                    ):
+                        # Other model expects this model to have a foreign key
+                        # For has_many/has_one, foreign_key is the FK column in THIS model
+                        fk = other_join.foreign_key or other_join.sql_expr
+                        if fk not in columns_added:
+                            select_cols.append(f"{fk} AS {fk}")
+                            columns_added.add(fk)
 
-        # Add dimension columns
-        # First, add all dimensions from this model (needed for filters/joins)
+        # Add only needed dimension columns
         for dimension in model.dimensions:
-            select_cols.append(f"{dimension.sql_expr} AS {dimension.name}")
+            if dimension.name in needed_dimensions and dimension.name not in columns_added:
+                # For time dimensions with granularity, apply DATE_TRUNC
+                if dimension.type == "time" and dimension.granularity:
+                    dim_sql = dimension.with_granularity(dimension.granularity)
+                else:
+                    dim_sql = dimension.sql_expr
+                select_cols.append(f"{dim_sql} AS {dimension.name}")
+                columns_added.add(dimension.name)
 
         # Then, add time dimensions with specific granularities
         for dim_ref, gran in dimensions:
@@ -528,7 +606,9 @@ class SQLGenerator:
                 # Apply time granularity (in addition to base column)
                 dim_sql = dimension.with_granularity(gran)
                 alias = f"{dim_name}__{gran}"
-                select_cols.append(f"{dim_sql} AS {alias}")
+                if alias not in columns_added:
+                    select_cols.append(f"{dim_sql} AS {alias}")
+                    columns_added.add(alias)
 
         # Add measure columns (raw, not aggregated in CTE)
         # Collect all measures needed for metrics
@@ -600,7 +680,7 @@ class SQLGenerator:
     def _has_fanout_joins(self, base_model_name: str, other_models: list[str]) -> dict[str, bool]:
         """Determine which models need symmetric aggregates due to fan-out.
 
-        When multiple one-to-many joins exist from the base model, measures from
+        When one-to-many joins exist from the base model, measures from
         the base model need symmetric aggregates to prevent double-counting.
 
         Args:
@@ -612,8 +692,9 @@ class SQLGenerator:
         """
         needs_symmetric = {}
 
-        # Check if there are multiple one-to-many relationships
+        # Check if there are any one-to-many relationships
         one_to_many_count = 0
+        many_to_one_models = []
 
         for other_model in other_models:
             try:
@@ -621,15 +702,25 @@ class SQLGenerator:
                 # Check if first hop is one-to-many
                 if join_path and join_path[0].relationship == "one_to_many":
                     one_to_many_count += 1
+                elif join_path and join_path[0].relationship == "many_to_one":
+                    # Track models with many-to-one from base perspective
+                    many_to_one_models.append(other_model)
             except (ValueError, KeyError):
                 pass
 
-        # If we have multiple one-to-many joins, the base model needs symmetric aggregates
-        needs_symmetric[base_model_name] = one_to_many_count > 1
+        # Base model needs symmetric aggregates if there are any one-to-many joins
+        needs_symmetric[base_model_name] = one_to_many_count > 0
 
-        # Other models generally don't need it (they're on the "many" side)
+        # Models on the "many" side of a many-to-one relationship also need symmetric
+        # aggregation if they're being joined (because from their perspective,
+        # they're creating fan-out for the "one" side)
         for other_model in other_models:
-            needs_symmetric[other_model] = False
+            if other_model in many_to_one_models:
+                # Check if the "one" side (base) has metrics - if so, it needs symmetric agg
+                # But we're checking from the perspective of this model, so mark False
+                needs_symmetric[other_model] = False
+            else:
+                needs_symmetric[other_model] = False
 
         return needs_symmetric
 
@@ -761,6 +852,7 @@ class SQLGenerator:
                                 primary_key=pk,
                                 agg_type=measure.agg,
                                 model_alias=f"{model_name}_cte",
+                                dialect=self.dialect,
                             )
                         else:
                             # Regular aggregation
